@@ -12,7 +12,7 @@
 #
 ##############################################################################
 """
-$Id: buildouthttp.py 115985 2010-08-28 07:08:32Z dobe $
+$Id: buildouthttp.py 116763 2010-09-23 12:16:01Z adamg $
 """
 
 import urllib2
@@ -87,6 +87,14 @@ class CredHandler(urllib2.HTTPBasicAuthHandler):
     """
 
     def http_error_401(self, req, fp, code, msg, headers):
+        try:
+            #python 2.6 introduces this attribute and fails the request on 5
+            #but since we're process global this fails on the 5th download
+            #there's no reset_retry_count(), so clear it here:
+            self.retried = 0
+        except AttributeError:
+            pass
+
         log.debug('getting url: %r' % req.get_full_url())
         try:
             res =  urllib2.HTTPBasicAuthHandler.http_error_401(
@@ -147,34 +155,37 @@ def install(buildout=None, pwd_path=None):
         return
 
 
-    reader = csv.reader(pwdsf)
-    auth_handler = CredHandler()
-    github_creds = get_github_credentials()
-    new_handlers = []
-    if github_creds:
-        new_handlers.append(GithubHandler(*github_creds))
+    try:
+        reader = csv.reader(pwdsf)
+        auth_handler = CredHandler()
+        github_creds = get_github_credentials()
+        new_handlers = []
+        if github_creds:
+            new_handlers.append(GithubHandler(*github_creds))
 
-    creds = []
-    for l, row in enumerate(reader):
-        if len(row) != 4:
-            raise RuntimeError(
-                "Authentication file cannot be parsed %s:%s" % (
-                    pwd_path, l+1))
-        realm, uris, user, password = (el.strip() for el in row)
-        creds.append((realm, uris, user, password))
-        log.debug('Added credentials %r, %r' % (realm, uris))
-        auth_handler.add_password(realm, uris, user, password)
-    if creds:
-        new_handlers.append(auth_handler)
-        download.url_opener = URLOpener(creds)
-    if new_handlers:
-        if urllib2._opener is not None:
-            handlers = urllib2._opener.handlers[:]
-            handlers[:0] = new_handlers
-        else:
-            handlers = new_handlers
-        opener = urllib2.build_opener(*handlers)
-        urllib2.install_opener(opener)
+        creds = []
+        for l, row in enumerate(reader):
+            if len(row) != 4:
+                raise RuntimeError(
+                    "Authentication file cannot be parsed %s:%s" % (
+                        pwd_path, l+1))
+            realm, uris, user, password = (el.strip() for el in row)
+            creds.append((realm, uris, user, password))
+            log.debug('Added credentials %r, %r' % (realm, uris))
+            auth_handler.add_password(realm, uris, user, password)
+        if creds:
+            new_handlers.append(auth_handler)
+            download.url_opener = URLOpener(creds)
+        if new_handlers:
+            if urllib2._opener is not None:
+                handlers = urllib2._opener.handlers[:]
+                handlers[:0] = new_handlers
+            else:
+                handlers = new_handlers
+            opener = urllib2.build_opener(*handlers)
+            urllib2.install_opener(opener)
+    finally:
+        pwdsf.close()
 
 class URLOpener(download.URLOpener):
 
